// Value_fct.cpp : Defines the entry point for the console application.
//

/*
Author: Klaus Reiner Schenk-Hopp (k.r.schenk-hoppe@leeds.ac.uk)
        http://www.schenk-hoppe.net
        
First version: February 20, 2005
Last update April 23, 2010
*/

#include "stdafx.h"

/**** Andreas Tupak's supporting code for large vectors and matrixes *****/

#include "ems_matrixT1.cpp"
#include "ems_vector.cpp"
#include "ems_funcobjects.cpp"
using namespace EMS;

#include <iostream> 
using namespace std;    

#include <cstdlib>
#include <cmath>


double utility_diff( double  _x, double _phi )
{
	return  _x >= 0.0  ?  _x  : _phi * _x; 
}

double utility( double  _x, double _sigma )
{
if (_sigma == 1)
    return  log(_x);
//  	  return  _x > 0.0  ?  log(_x) : -1000000000000000.0;
else {
   if (_sigma > 1)
  	  return  _x > 0.0  ?   pow(_x,1.0-_sigma)/(1.0-_sigma) : -10000000000.0;
   else  
    return  pow(_x,(1.0-_sigma))/(1.0-_sigma);
  }
}

double output( double  _a, double  _x )
{
	return  _a * pow(_x,0.5);
}


typedef struct {
     long type;   /* type */
     long mrows;  /* row dimension */
     long ncols;  /* column dimension */
     long imagf;  /* flag indicating imag part */
	 size_t namlen; /* name length (including NULL) */
     // long namlen; /* name length (including NULL) */
} Fmatrix;

/*
 * savemat - C language routine to save a matrix in a MAT-file.
 *
 * Here is an example that uses 'savemat' to save two matrices to disk,
 * the second of which is complex:
 *
 *  FILE *fp;
 *  double xyz[1000], ar[1000], ai[1000];
 *  fp = fopen("foo.mat","wb");
 *  savemat(fp, 2000, "xyz", 2, 3, 0, xyz, (double *)0);
 *  savemat(fp, 2000, "a", 5, 5, 1, ar, ai);
 *      fclose(fp);
 *
 * Author J.N. Little 11-3-86
 */

void
savemat( FILE *  fp,  int  type,  char *  pname,  int  mrows,  int  ncols,
         int  imagf,  double *  preal, double *  pimag)
     /* FILE *fp;       File pointer */
     /* int type;       Type flag: Normally 0 for PC, 1000 for Sun, Mac, and     */
     /*                 Apollo, 2000 for VAX D-float, 3000 for VAX G-float    */
     /*                 Add 1 for text variables.    */
     /*                 See LOAD in reference section of guide for more info. */
     /* char *pname;    pointer to matrix name */
     /* int mrows;      row dimension */
     /* int ncols;      column dimension */
     /* int imagf;      imaginary flag */
     /* double *preal;  pointer to real data */
     /* double *pimag;  pointer to imag data */
{
    Fmatrix x;
    int mn;

    x.type = type;
    x.mrows = mrows;
    x.ncols = ncols;
    x.imagf = imagf;
    x.namlen = strlen(pname) + 1;
    mn = x.mrows * x.ncols;

    fwrite(&x, sizeof(Fmatrix), 1, fp);
    fwrite(pname, sizeof(char), x.namlen, fp);
    fwrite(preal, sizeof(double), mn, fp);
    if (imagf) {
         fwrite(pimag, sizeof(double), mn, fp);
    }
}


/********************************************/

int _tmain(int argc, _TCHAR* argv[])
{
    typedef  MatrixT1RM<double>  Mat_t;
    typedef  VectorT1<double>    Vec_t;

    unsigned int  Max_coord(0),
                  V_iter   (0);

    double  V_xmax(0.0);
    double  V_beta(0.95);
    double  V_alpha(1.0);
    double  phi(2.25);
    double  sigma(2.0);
    double  A_prod(1.0);
	string  FileName("");

//  Discounting rate beta
	V_beta = 0.95;

//  Weight parameter alpha
	V_alpha = 0.9;

// utility function parameter   
	sigma = 0.5;
 
// standard loss aversion (phi = 2.25), habit (phi = 1)   
	phi = 2.25;

// production function parameter A_prod * f(k)
	A_prod = 1.5;

//    cerr << "Enter Filename (without suffix): ";
//    cin >> FileName;

	FileName = "V_alpha_09_sigma_05_loss_A_15";
	FileName  +=  ".mat";

//    cerr << "Maximum x,y-range: ";
//    cin >>  V_xmax;

	V_xmax = A_prod * A_prod; // ensures that A_prod * sqrt(V_max) <= V_max

    //cerr << "Grid size (1001): ";
    //cin >>  Max_coord;

	Max_coord = 1001;

    Mat_t  Vold(Max_coord, Max_coord),
           Vnew(Max_coord, Max_coord);

    Vec_t  XCOORD(Max_coord),
           YCOORD(Max_coord);

//    cerr << "Iteration steps: ";
//    cin >>  V_iter;

	V_iter = 100;   

    Vec_t  improvement(V_iter);
    

//  for_each_in( Vold, Clear()); // redundant

    double const  myp_scale (V_xmax / (double)(Max_coord));
    double ini_value_V (0);

	if (sigma > 1)
 	    double ini_value_V (100.0*utility(myp_scale,sigma));
	
	for_each_in( Vnew, SetTo<double>(ini_value_V) ); 
	{
	double _tmp(V_xmax / (double)(Max_coord));
    for( unsigned int  _i(0); _i < Max_coord; ++ _i )
       {
           XCOORD[ _i ]  =  YCOORD[ _i ]  =  _tmp  *  _i;
       }
	}
    for( unsigned int  t(0); t < V_iter; ++ t )
    {
        cerr << (V_iter - t - 1) << "iterations to go" << endl;
        
		unsigned int l_mini (0);
		//if (sigma < 1)
		//	l_mini = 0;

        for( unsigned int  k(0); k < Max_coord; ++ k )
        {
            double const &  x_1 (XCOORD[ k ]); 
            
            // x -> x_1, y -> x_2, xp1 -> y_1, yp1 -> y_2

            for( unsigned int  l(l_mini); l < Max_coord; ++ l )  // l(1) for the case sigma > 1.
            {
                double          maxiF(-1);
				
				if (sigma >= 1)
 					maxiF = -100000000000.0;
				
				double const &  x_2    (YCOORD[ l ]),
                                y_1  (x_2);  // i.e. y^1 = x^2 
              
				// cerr << "x =" << x_1 << "," "y =" << x_2 << endl;
               /*  unsigned int  myp_max((unsigned int) (productionF( xp1, case_f ) / V_xmax * (Max_coord + 3)));
                if( myp_max > Max_coord ) myp_max  =  Max_coord;
                for( unsigned int  myp(0); myp < myp_max; ++ myp )   */      


				for( unsigned int  myp(0); myp < Max_coord; ++ myp )        // would myp < Max_coord * output(y_1) be okay ??? 
                {
                    double  y_2 (myp_scale * myp);
                    
                   if( y_2 <= output(A_prod,y_1) )
                    {
                        double  cons_diff = output(A_prod,y_1) - y_2 - (output(A_prod,x_1) - x_2);
                        double  new_val = (1.0 - V_alpha)*utility(output(A_prod,y_1) - y_2,sigma) + V_alpha*utility_diff( cons_diff,phi ) + V_beta * Vold( l, myp );
                              // new_val((1.0 - V_alpha)*(pow( 0.5, -1.0))*(pow((val - y_2), 0.5)) + V_alpha*UFkt( _arg ) + V_beta * Vold( l, myp ));
                        if( new_val > maxiF ) maxiF  =  new_val;
                    }
                }
                Vnew( k, l )  =  maxiF;
            }
        }

        // max(max(vneu-vold))
        double  _change(-1.0);
        
        Mat_t::iterator  old_Bi(Vold.begin()),
                         old_Ei(Vold.  end()),
                         new_Ei(Vnew.  end());
                         
        while( old_Ei != old_Bi )
        {
            -- old_Ei;
            -- new_Ei;
            
            double  _tmp(* old_Ei);
            
            if( (_tmp  =  abs(_tmp - (* old_Ei  =  * new_Ei))) > _change ) _change  =  _tmp;
        }
        improvement[ t ]  =  _change;
    }


//    FILE *  fp_mat  =  fopen( FileName.c_str(), "wb" );
    FILE *  fp_mat;
    fopen_s( &fp_mat, FileName.c_str(), "wb" );

    double  _tmp;
    savemat( fp_mat, 0, "beta_value" , 1,         1, 0, &               V_beta, 0);
    savemat( fp_mat, 0, "alpha_value", 1,        1, 0, &               V_alpha, 0);
    savemat( fp_mat, 0, "phi"       , 1,        1, 0, &               phi, 0);
    savemat( fp_mat, 0, "sigma"      , 1,        1, 0, &               sigma, 0);
    savemat( fp_mat, 0, "A_prod"      , 1,        1, 0, &               A_prod, 0);
    savemat( fp_mat, 0, "iterations" , 1,         1, 0, & (_tmp  =  V_iter   ), 0);
    savemat( fp_mat, 0, "max_coord"  , 1,         1, 0, & (_tmp  =  Max_coord), 0);
    savemat( fp_mat, 0, "x"          , 1, Max_coord, 0, &          XCOORD[ 0 ], 0);
    savemat( fp_mat, 0, "y"          , 1, Max_coord, 0, &          YCOORD[ 0 ], 0);
    savemat( fp_mat, 0, "improvement", 1,    V_iter, 0, &     improvement[ 0 ], 0);


    for( unsigned int mxp(0); mxp < Max_coord; ++ mxp) 
        for( unsigned int myp(0); myp < Max_coord; ++ myp) 
            Vold( myp, mxp )  =  Vnew( mxp, myp );

    savemat( fp_mat, 0, "V", Max_coord, Max_coord, 0, & Vold( 0, 0 ), 0 );


    fclose(fp_mat);
 
    return  0;  
}
